#include "stack.h"
#include <stdlib.h>


struct stack
{
    stack_value_t *data;
    size_t         size;
};

stack_t *stack_create()
{
    stack_t *stack = malloc(sizeof(stack_t));
    if (stack)
        stack_construct(stack);
    return stack;
}

void stack_delete(stack_t *stack)
{
    stack_destroy(stack);
    free(stack);
}

void stack_construct(stack_t *self)
{
    self->data = NULL;
    self->size = 0;
}

void stack_destroy(stack_t *self)
{
    if (self->data)
    {
        free(self->data);
        self->data = NULL;
    }
    self->size = 0;
}

size_t stack_size(const stack_t *self)
{
    return self->size;
}

void stack_push(stack_t *self, stack_value_t val)
{
    stack_value_t *newdata = realloc(self->data, 
            (self->size+1)*sizeof(stack_value_t));
    if (newdata)
    {
        self->data = newdata;
        self->data[self->size] = val;
        self->size++;
    }
}

stack_value_t stack_pop(stack_t *self)
{
    if (!self->size) 
        return 0;
    stack_value_t val = stack_peek(self);
    size_t newsize = self->size-1;
    stack_value_t *newdata = realloc(self->data, 
            newsize*sizeof(stack_value_t));
    if (newdata || newsize == 0)
        self->data = newdata;
    self->size = newsize;
    return val;
}

stack_value_t stack_peek(const stack_t *self)
{
    return (self->size != 0) ? self->data[self->size-1] : 0;
}

