#include "person.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

typedef void (*person_delete_f)(person_t *self);
typedef void (*person_print_f)(const person_t *self);

struct person_vtab
{
    person_delete_f delete;
    person_print_f  print;
};

typedef struct person_vtab person_vtab_t;

static const person_vtab_t person_vtab =
{
    person_delete,
    person_print
};

void person_vdelete(person_t *person)
{
    ((person_vtab_t *) person->vtab)->delete(person);
}

void person_vprint(const person_t *person)
{
    ((person_vtab_t *) person->vtab)->print(person);
}


person_t *person_create(const char *name)
{
    person_t *person = malloc(sizeof(person_t));
    person_construct(person, name);
    return person;
}

void person_delete(person_t *person)
{
    person_destruct(person);
    free(person);
}

void person_construct(person_t *person, const char *name)
{
    person->vtab = &person_vtab; 
    person->name = malloc(strlen(name)+1);
    strcpy(person->name, name);
}

void person_destruct(person_t *person)
{
    free(person->name);
    person->name = NULL;
}

void person_print(const person_t *person)
{
    printf("name: %s\n", person->name);
}

// Student

typedef person_vtab_t student_vtab_t;

static const student_vtab_t student_vtab = 
{
    (person_delete_f) student_delete,
    (person_print_f)  student_print
};

student_t *student_create(const char *name, int number)
{
    student_t *student = malloc(sizeof(student_t));
    student_construct(student, name, number);
    return student;
}

void student_delete(student_t *student)
{
    student_destruct(student);
    free(student);
}

void student_construct(student_t *student, const char *name, int number)
{
    person_construct(&student->super, name);
    student->super.vtab = &student_vtab;
    student->number = number;
}

void student_destruct(student_t *student)
{
    person_destruct(&student->super);
}

void student_print(const student_t *student)
{
    person_print(&student->super);
    printf("number: %d\n", student->number);
}
